import cockpit from "cockpit";
import React from "react";
import { ErrorMessage, NotificationProvider, Notifications } from "./app.components.jsx";
import { DOWNCLOCK_CPU_ID, RevPiConfigService } from "./revpi-config/RevPiConfigService.js";
import { Page, PageSection, PageSectionVariants, Grid, GridItem } from "@patternfly/react-core";
import {
    RevPiDropdown,
    RevPiMultiToggle,
    RevPiSwitch,
    RevPiToggle
} from "./common.components.jsx";
import { PictoryCard } from "./features/PictoryCard.jsx";
import { NetworkCard } from "./features/NetworkCard.jsx";
import { NodeRedCard } from "./features/NodeRedCard.jsx";
import { SystemConfigurationCard } from "./features/SystemConfigurationCard.jsx";
import { RunTimeCard } from "./features/RunTimeCard.jsx";

const _ = cockpit.gettext;

export class Application extends React.Component {
    constructor (props) {
        super(props);
        this.state = {
            isLoading: true,
            features: new Map()
        };
        this.service = new RevPiConfigService();
    }

    async handleSwitchClick (id, isEnabled, isLoading) {
        if (id === DOWNCLOCK_CPU_ID) {
            return this.handleDownClockCpu(isEnabled);
        }

        this.updateFeatureLoadingState(id, isLoading);
        try {
            await this.service.toggleConfiguration(id, isEnabled);
            await this.loadAllFeatures();
        } catch {
            this.updateFeatureLoadingState(id, !isLoading);
            this.setFeatureError(id);
        }
    }

    async handleDownClockCpu (value) {
        this.updateFeatureLoadingState(DOWNCLOCK_CPU_ID, false);
        const doEnable = value !== "MAX";
        try {
            await this.service.updateCpuRate(doEnable, { newRate: value });
            await this.loadAllFeatures();
        } catch {
            this.setFeatureError(DOWNCLOCK_CPU_ID);
            this.updateFeatureLoadingState(DOWNCLOCK_CPU_ID, true);
        }
    }

    updateFeatureLoadingState (id, isLoading) {
        const { features } = this.state;
        features.get(id).isLoading = !isLoading;
        this.setState({ features });
    }

    setFeatureError (id) {
        const { features } = this.state;
        features.get(id).hasError = true;
        this.setState({ features });
    }

    getFeatureData () {
        return this.service.getRevpiConfigStatus();
    }

    async loadAllFeatures () {
        const features = await this.getFeatureData();
        this.setState({
            features: new Map(features.map(x => [x.id, x])),
            isLoading: false
        });
    }

    load () {
        this.setState({
            error: undefined,
            isLoading: true
        });

        const permission = cockpit.permission({ admin: true });

        permission.addEventListener("changed", async () => {
            if (permission.allowed) {
                try {
                    await this.loadAllFeatures();
                    this.setState({
                        error: undefined,
                        isLoading: false
                    });
                } catch (error) {
                    console.error(_("There was an error while loading all features. ") + error);
                    this.setState({
                        error: error.message,
                        isLoading: false
                    });
                }
            } else {
                this.setState({
                    error: "access-denied",
                    isLoading: false
                });
            }
        });
    }

    componentDidMount () {
        this.load();
    }

    render () {
        if (this.state.isLoading || this.state.error) {
            return <ErrorMessage loading={this.state.isLoading} error={this.state.error} />;
        }

        const switchHandler = (id, isEnabled, isLoading) => this.handleSwitchClick(id, isEnabled, isLoading);
        const getRevpiSwitch = id => <RevPiSwitch item={this.state.features.get(id)} switchHandler={switchHandler} />;
        const getRevPiToggle = id => <RevPiToggle item={this.state.features.get(id)} switchHandler={switchHandler} />;
        const nodeRedFeatureItem = this.state.features.get("nodered");

        const featureControlFactory = {
            BoundSwitch: ({ id }) => <RevPiSwitch item={this.state.features.get(id)} switchHandler={switchHandler} />,
            BoundToggle: ({ id }) => <RevPiToggle item={this.state.features.get(id)} switchHandler={switchHandler} />,
            BoundMultiToggle: ({ id }) => <RevPiMultiToggle item={this.state.features.get(id)} switchHandler={switchHandler} />,
            BoundRevPiDropdown: ({ id }) => <RevPiDropdown item={this.state.features.get(id)} switchHandler={switchHandler} />
        };

        return (
            <NotificationProvider>
                <Page>
                    <PageSection variant={PageSectionVariants.default}>
                        <Notifications />
                        <Grid className='ct-system-overview' hasGutter>
                            <GridItem md={4} sm={12}>
                                <PictoryCard getRevpiSwitch={getRevpiSwitch} />
                            </GridItem>

                            <GridItem md={4} sm={12}>
                                <NodeRedCard item={nodeRedFeatureItem} getRevpiSwitch={getRevpiSwitch} getRevPiToggle={getRevPiToggle} />
                            </GridItem>

                            <GridItem md={4} sm={12}>
                                <RunTimeCard getRevpiSwitch={getRevpiSwitch} getRevPiToggle={getRevPiToggle} />
                            </GridItem>

                            <GridItem md={6} sm={12}>
                                <SystemConfigurationCard featureControlFactory={featureControlFactory} />
                            </GridItem>

                            <GridItem md={6} sm={12}>
                                <NetworkCard getRevpiSwitch={getRevpiSwitch} getRevPiToggle={getRevPiToggle} />
                            </GridItem>
                        </Grid>
                    </PageSection>
                </Page>
            </NotificationProvider>
        );
    }
}
