import React, { useState } from "react";
import cockpit from "cockpit";
import { Alert, Card, CardBody, CardHeader, Progress } from "@patternfly/react-core";
import { canInstallPackage, installPackage, updatePackages, useInstallationStatus } from "./package-utils.js";
import { useNotifications } from "../app.components.jsx";
import { Button } from "@patternfly/react-core/dist/esm/components/Button";

const _ = cockpit.gettext;

function useInstallationState (featureName) {
    const [installationState, setInstallationState] = useState({ state: null, text: null, progress: 0 });

    const updateInstallationState = (newState) => {
        const stateInfo = { state: newState, text: "", progress: 0 };

        switch (newState) {
        case "updating":
            stateInfo.text = _("Updating Data Sources");
            stateInfo.progress = 10;
            break;
        case "checking":
            stateInfo.text = _("Checking Compatibility");
            stateInfo.progress = 30;
            break;

        case "installing":
            stateInfo.text = _("Installing ") + featureName;
            stateInfo.progress = 60;
            break;
        case "success":
            stateInfo.text = `${featureName} has been successfully installed.`;
            stateInfo.progress = 100;
            break;
        default:
            stateInfo.text = "Unknown state.";
            stateInfo.progress = 0;
        }
        setInstallationState(stateInfo);
    };

    return [installationState, updateInstallationState];
}

export function InstallFeatureCard ({ featureName, packageName, children }) {
    const [isInstalled, handleInstallationComplete] = useInstallationStatus(packageName);
    const [canInstall, setCanInstall] = useState(false);
    const [isInstalling, setIsInstalling] = useState(false);
    const [hasError, setHasError] = useState(false);
    const [installationState, updateInstallationState] = useInstallationState(featureName);
    const { addNotification } = useNotifications();
    const notificationTitle = _("Installation successful");
    const notificationText = _("You have successfully installed ");
    const handleInstallClick = async () => {
        setIsInstalling(true);
        try {
            updateInstallationState("updating");
            await updatePackages();
            updateInstallationState("checking");
            const installable = await canInstallPackage(packageName);
            setCanInstall(installable);
            if (installable) {
                updateInstallationState("installing");
                await installPackage(packageName);
                handleInstallationComplete();
                addNotification(notificationTitle, notificationText + featureName);
            }
        } catch (error) {
            setHasError(true);
        } finally {
            setIsInstalling(false);
        }
    };

    if (isInstalled) return <>{children}</>;

    if (hasError) return <div>{_("An error occurred installing ") + featureName}</div>;

    if (isInstalling) {
        return (
            <Card is>
                <CardHeader>
                    <div className='dots' style={{ fontWeight: "550", fontStyle: "italic" }}> {_("Installation in progress")}</div>
                </CardHeader>
                <CardBody>
                    <Progress
                        value={installationState.progress}
                        helperText={installationState.text}
                    />
                </CardBody>
            </Card>
        );
    }

    return (
        <Card>
            <CardHeader>
                <Alert
                    variant='info' isInline isPlain
                    title={`${featureName} ${_("is not installed. To  install it, use the button below.")}`}
                />
            </CardHeader>
            <CardBody>
                <Button
                    onClick={handleInstallClick}
                    disabled={!canInstall}
                >
                    {`${_("Install")} ${featureName}`}
                </Button>
            </CardBody>
        </Card>
    );
}
